import numpy as np
from scipy.integrate import quad
from .boys import *

def phi_n_unnorm(t,n,alphaa,beta,X):
    if(n==0):
        return 1
    elif(n>0):
        return (t*phi_n(t,n-1,alphaa,beta,X)-beta[n-1]*phi_n(t,n-2,alphaa,beta,X)-alphaa[n-1]*phi_n(t,n-1,alphaa,beta,X))
    else:
        return 0

def phi_n(t,n,alphaa,beta,X):
    if(n==0):
        return 1/np.sqrt(F_n(0,X))
    elif(n>0):
        return (t*phi_n(t,n-1,alphaa,beta,X)-beta[n-1]*phi_n(t,n-2,alphaa,beta,X)-alphaa[n-1]*phi_n(t,n-1,alphaa,beta,X))/beta[n]
    else:
        return 0
    
def alphaa_func(t,n,alphaa,beta,X):
    return phi_n(t**2,n,alphaa,beta,X)*t**2*phi_n(t**2,n,alphaa,beta,X)*np.exp(-X*t**2)

def beta_func(t,n,alphaa,beta,X):
    return phi_n_unnorm(t**2,n,alphaa,beta,X)*phi_n_unnorm(t**2,n,alphaa,beta,X)*np.exp(-X*t**2)

def get_weights_roots(points,X):

    alphaa = np.zeros(points)
    beta = np.zeros(points)

    for n in range(points):
        beta[n] = np.sqrt(quad(beta_func,0,1,args=(n,alphaa,beta,X))[0])
        alphaa[n] = quad(alphaa_func,0,1,args=(n,alphaa,beta,X))[0]

    M = np.zeros((points,points))
    for i in range(points):
        M[i][i] = alphaa[i]
        if(i>0):
            M[i][i-1] = beta[i]
        if(i<points-1):
            M[i][i+1] = beta[i+1]     
            
    val,vec = np.linalg.eigh(M)
    w = np.zeros(points)
    for i in range(points):
        w[i] = vec[0][i]**2*F_n(0,X)
    roots = np.sqrt(val)
    return w,roots


# ## DKR4

# Se define la relación de recurrencia
# \begin{eqnarray}
# I_{i+1,j,k,l} &=& \left( X_{PA} - \frac{\alpha}{p}X_{PQ}s^2 \right) I_{ijkl} + \frac{1}{2p} \left(1-\frac{\alpha}{p} s^2 \right) (iI_{i-1,j,k,l} + jI_{i,j-1,k,l}) + \frac{s^2}{2(p+q)}(kI_{i,j,k-1,l}+lI_{i,j,k,l-1})\\
# I_{i,j,k+1,l} &=& \left( X_{QC} + \frac{\alpha}{q}X_{PQ}s^2 \right) I_{ijkl} + \frac{1}{2q} \left(1-\frac{\alpha}{q} s^2 \right) (kI_{i,j,k-1,l} + lI_{i,j,k,l-1}) + \frac{s^2}{2(p+q)}(iI_{i-1,j,k,l}+jI_{i,j-1,k,l})
# \end{eqnarray}

def I_DKR4(s,i,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz):    

    if(i<0 or j<0 or k<0 or l<0):
        return 0
    elif(i>0):
        return ((pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])-alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR4(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairAB.exp[ii])*(1-alpha/pairAB.exp[ii]*s**2)*((i-1)*I_DKR4(s,i-2,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz)+j*I_DKR4(s,i-1,j-1,k,l,pairAB,pairCD,alpha,ii,jj,xyz)) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(k*I_DKR4(s,i,j,k-1,l,pairAB,pairCD,alpha,ii,jj,xyz) + l*I_DKR4(s,i,j,k,l-1,pairAB,pairCD,alpha,ii,jj,xyz))
    elif(j>0):
        return ((pairAB.coord[ii][xyz]-pairAB.g_b.coord[xyz])-alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR4(s,i,j-1,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairAB.exp[ii])*(1-alpha/pairAB.exp[ii]*s**2)*((j-1)*I_DKR4(s,i,j-2,k,l,pairAB,pairCD,alpha,ii,jj,xyz)+i*I_DKR4(s,i-1,j-1,k,l,pairAB,pairCD,alpha,ii,jj,xyz)) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(k*I_DKR4(s,i,j,k-1,l,pairAB,pairCD,alpha,ii,jj,xyz) + l*I_DKR4(s,i,j,k,l-1,pairAB,pairCD,alpha,ii,jj,xyz))
    elif(k>0):
        return ((pairCD.coord[jj][xyz]-pairCD.g_a.coord[xyz])+alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR4(s,i,j,k-1,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairCD.exp[jj])*(1-alpha/pairCD.exp[jj]*s**2)*((k-1)*I_DKR4(s,i,j,k-2,l,pairAB,pairCD,alpha,ii,jj,xyz)+l*I_DKR4(s,i,j,k-1,l-1,pairAB,pairCD,alpha,ii,jj,xyz)) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(i*I_DKR4(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + j*I_DKR4(s,i,j-1,k,l,pairAB,pairCD,alpha,ii,jj,xyz))
    elif(l>0):
        return ((pairCD.coord[jj][xyz]-pairCD.g_b.coord[xyz])+alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR4(s,i,j,k,l-1,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairCD.exp[jj])*(1-alpha/pairCD.exp[jj]*s**2)*((l-1)*I_DKR4(s,i,j,k,l-2,pairAB,pairCD,alpha,ii,jj,xyz)+k*I_DKR4(s,i,j,k-1,l-1,pairAB,pairCD,alpha,ii,jj,xyz)) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(i*I_DKR4(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + j*I_DKR4(s,i,j-1,k,l,pairAB,pairCD,alpha,ii,jj,xyz))
    else:
        return np.exp(-pairAB.alpha[ii]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])**2)*np.exp(-pairCD.alpha[jj]*(pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])**2)


# Finalmente
# \begin{equation}
# g_{abcd} = \frac{2\pi^{5/2}}{pq\sqrt{p+q}} \sum_{k=1}^\gamma w_k I_x(s_k^2) I_y(s_k^2) I_z(s_k^2)
# \end{equation}

def DKR4(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    integral = 0.0
                    
                    for ii in range(pairAB.k):
                        aux2 = 0.0
                        for jj in range(pairCD.k):
                            
                            alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])
                            Rpq = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
                    
                            points = int((lax+lay+laz+lbx+lby+lbz+lcx+lcy+lcz+ldx+ldy+ldz)/2)+1
                            w,s = get_weights_roots(points,alpha*Rpq**2)
                            
                            aux1 = 0
                            for k in range(points):
                                aux1 += w[k]*I_DKR4(s[k],lax,lbx,lcx,ldx,pairAB,pairCD,alpha,ii,jj,0)*I_DKR4(s[k],lay,lby,lcy,ldy,pairAB,pairCD,alpha,ii,jj,1)*I_DKR4(s[k],laz,lbz,lcz,ldz,pairAB,pairCD,alpha,ii,jj,2)
                            aux1 = 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*aux1                            
                            aux2 += aux1*pairCD.coef[jj]
                        integral += aux2*pairAB.coef[ii]
                        
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result    
                


# ## DKR2

# Se define la relación de recurrencia
# \begin{eqnarray}
# I_{i+1,0,k,0} &=& \left( X_{PA} - \frac{\alpha}{p}X_{PQ}s^2 \right) I_{i0k0} + \frac{1}{2p} \left(1-\frac{\alpha}{p} s^2 \right) (iI_{i-1,0,k,0}) + \frac{s^2}{2(p+q)}(kI_{i,j,k-1,l})\\
# I_{i,0,k+1,0} &=& \left( X_{QC} + \frac{\alpha}{q}X_{PQ}s^2 \right) I_{i0k0} + \frac{1}{2q} \left(1-\frac{\alpha}{q} s^2 \right) (kI_{i,0,k-1,0}) + \frac{s^2}{2(p+q)}(iI_{i-1,j,k,l})
# \end{eqnarray}

def I_DKR2(s,i,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz):    

    if(i<0 or j<0 or k<0 or l<0):
        return 0
    elif(i>0):
        return ((pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])-alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR2(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairAB.exp[ii])*(1-alpha/pairAB.exp[ii]*s**2)*(i-1)*I_DKR2(s,i-2,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*k*I_DKR2(s,i,j,k-1,l,pairAB,pairCD,alpha,ii,jj,xyz)
    elif(k>0):
        return ((pairCD.coord[jj][xyz]-pairCD.g_a.coord[xyz])+alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR2(s,i,j,k-1,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairCD.exp[jj])*(1-alpha/pairCD.exp[jj]*s**2)*(k-1)*I_DKR2(s,i,j,k-2,l,pairAB,pairCD,alpha,ii,jj,xyz) + s**2/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*i*I_DKR2(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz)
    else:
        return np.exp(-pairAB.alpha[ii]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])**2)*np.exp(-pairCD.alpha[jj]*(pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])**2)


# Se añade relación de recurrencia horizontal

def HRR_DKR2(pairAB,pairCD,i,j,k,l,N):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR_DKR2(pairAB,pairCD,i,j,kp1,lm1,N) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])*HRR_DKR2(pairAB,pairCD,i,j,k,lm1,N)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR_DKR2(pairAB,pairCD,ip1,jm1,k,l,N) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR_DKR2(pairAB,pairCD,i,jm1,k,l,N)
    
    integral = 0.0
    for ii in range(pairAB.k):
        aux2 = 0.0
        for jj in range(pairCD.k):
                    
            alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])
            Rpq = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
                
            points = int((i[0]+i[1]+i[2]+j[0]+j[1]+j[2]+k[0]+k[1]+k[2]+l[0]+l[1]+l[2])/2)+1
            w,s = get_weights_roots(points,alpha*Rpq**2)
                        
            aux1 = 0
            for kk in range(points):
                aux1 += w[kk]*I_DKR2(s[kk],i[0],j[0],k[0],l[0],pairAB,pairCD,alpha,ii,jj,0)*I_DKR2(s[kk],i[1],j[1],k[1],l[1],pairAB,pairCD,alpha,ii,jj,1)*I_DKR2(s[kk],i[2],j[2],k[2],l[2],pairAB,pairCD,alpha,ii,jj,2)
            aux1 = 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*aux1                            
            aux2 += aux1*pairCD.coef[jj]
        integral += aux2*pairAB.coef[ii]
    
    return integral


def DKR2(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
    
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_DKR2(pairAB,pairCD,i,j,k,l,0)                
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result           

# ## DKR1

# Se define la relación de recurrencia
# \begin{eqnarray}
# I_{i+1,0,0,0} &=& \left( X_{PA} - \frac{\alpha}{p}X_{PQ}s^2 \right) I_{i000} + \frac{1}{2p} \left(1-\frac{\alpha}{p} s^2 \right) (iI_{i-1,0,0,0})\\
# \end{eqnarray}

def I_DKR1(s,i,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz):    

    if(i<0 or j<0 or k<0 or l<0):
        return 0
    elif(i>0):
        return ((pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])-alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*s**2)*I_DKR1(s,i-1,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz) + 1/(2*pairAB.exp[ii])*(1-alpha/pairAB.exp[ii]*s**2)*(i-1)*I_DKR1(s,i-2,j,k,l,pairAB,pairCD,alpha,ii,jj,xyz)
    else:
        return np.exp(-pairAB.alpha[ii]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])**2)*np.exp(-pairCD.alpha[jj]*(pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])**2)


def trans_elec_DKR1(i,j,k,l,pairAB,pairCD,ii,jj):

    integral = 0.0
    
    for xyz in range(3):
        if(i[xyz]<0 or k[xyz]<0):
            return 0
        elif(k[xyz]!=0):
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1 
            km2 = k[:]
            km2[xyz] = km2[xyz] - 2 
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1         
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1        
            return -(pairAB.ab[ii][0]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])+pairCD.ab[jj][1]*(pairCD.g_c.coord[xyz]-pairCD.g_d.coord[xyz]))/pairCD.exp[jj]*trans_elec_DKR1(i,j,km1,l,N,pairAB,pairCD) + i[xyz]/(2*pairCD.exp[jj])*trans_elec_DKR1(im1,j,km1,l,N,pairAB,pairCD) + (k[xyz]-1)/(2*pairCD.exp[jj])*trans_elec_DKR1(i,j,km2,l,N,pairAB,pairCD) - pairAB.exp[ii]/pairCD.exp[jj]*trans_elec_DKR1(ip1,j,km1,l,N,pairAB,pairCD)
    return I_DKR1(i,j,k,l,pairAB,pairCD,ii,jj)

def HRR_DKR1(pairAB,pairCD,i,j,k,l):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR_DKR1(pairAB,pairCD,i,j,kp1,lm1) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])*HRR_DKR1(pairAB,pairCD,i,j,k,lm1)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR_DKR1(pairAB,pairCD,ip1,jm1,k,l) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR_DKR1(pairAB,pairCD,i,jm1,k,l)
    
    integral = 0.0
    for ii in range(pairAB.k):
        aux2 = 0.0
        for jj in range(pairCD.k):
                    
            alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])
            Rpq = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
                
            points = int((i[0]+i[1]+i[2]+j[0]+j[1]+j[2]+k[0]+k[1]+k[2]+l[0]+l[1]+l[2])/2)+1
            w,s = get_weights_roots(points,alpha*Rpq**2)
                        
            aux1 = 0
            for kk in range(points):
                aux1 += w[kk]*I_DKR1(s[kk],i[0],j[0],k[0],l[0],pairAB,pairCD,alpha,ii,jj,0)*I_DKR1(s[kk],i[1],j[1],k[1],l[1],pairAB,pairCD,alpha,ii,jj,1)*I_DKR1(s[kk],i[2],j[2],k[2],l[2],pairAB,pairCD,alpha,ii,jj,2)
            aux1 = 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*aux1                            
            aux2 += aux1*pairCD.coef[jj]
        integral += aux2*pairAB.coef[ii]
    
    return integral

def DKR1(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
    
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_DKR1(pairAB,pairCD,i,j,k,l)                
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result



import numpy as np
from .boys import *

# ## MD4

# Se definen las relaciones de recurrencia para E (aquí se expresan las de $X$, pero hay otras equivalentes para $Y$ y $Z$).
# \begin{eqnarray}
# E_t^{i_x+1,j_x} &=& \frac{1}{2p} E^{i_x j_x}_{t-1} + X_{PA} E_t^{i_x j_x}+(t+1)E^{i_x j_x}_{t+1}\\
# E_t^{i_x,j_x+1} &=& \frac{1}{2p} E^{i_xj_x}_{t-1} + X_{PB} E_t^{i_xj_x}+(t+1)E^{i_xj_x}_{t+1}
# \end{eqnarray}
# que llegan hasta
# \begin{equation}
# E_{0}^{00} = K_{ab}^x
# \end{equation}
# 
# Estas provienen de la expansión del producto de Gaussianas cartesianas en gaussianas Hermitianas.
# \begin{equation}
# \Omega_{i_x,j_x} = g_{i_x} g_{j_x} = \sum_t^{i_x+j_x} E_{t}^{i_x,j_x} \Lambda_{t}
# \end{equation}

def compute_E(t,la,lb,pair,i,xyz):
    
    if(t<0):
        return 0
    elif(t>la+lb):
        return 0
    elif(la>0):
        E_im1j_tm1 = compute_E(t-1,la-1,lb,pair,i,xyz)
        E_im1j_t = compute_E(t,la-1,lb,pair,i,xyz)
        E_im1j_tp1 = compute_E(t+1,la-1,lb,pair,i,xyz)
        return 1/(2*pair.exp[i])*E_im1j_tm1 + (pair.coord[i][xyz]-pair.g_a.coord[xyz])*E_im1j_t + (t+1)*E_im1j_tp1
    elif(lb>0):
        E_ijm1_tm1 = compute_E(t-1,la,lb-1,pair,i,xyz)
        E_ijm1_t = compute_E(t,la,lb-1,pair,i,xyz)
        E_ijm1_tp1 = compute_E(t+1,la,lb-1,pair,i,xyz)    
        return 1/(2*pair.exp[i])*E_ijm1_tm1 + (pair.coord[i][xyz]-pair.g_b.coord[xyz])*E_ijm1_t + (t+1)*E_ijm1_tp1
    elif(t==0 and la==0 and lb==0):
        return np.exp(-pair.alpha[i]*(pair.g_b.coord[xyz]-pair.g_a.coord[xyz])**2)


# Se definen las relaciones de recurrencia
# \begin{eqnarray}
# R_{000}^n &=& -(2\alpha)^n \frac{2 \pi^{5/2}}{pq \sqrt{p+q}} F_{n}(\alpha R_{PQ}^2) \\
# R^n_{t_x+1,t_y,t_z} &=& t_x R^{n+1}_{t_x-1,t_y,t_z} + X_{PQ} R^{n+1}_{t_x,t_y,t_z} \\
# R^n_{t_x,t_y+1,t_z} &=& t_y R^{n+1}_{t_x,t_y-1,t_z} + Y_{PQ} R^{n+1}_{t_x,t_y,t_z} \\
# R^n_{t_x,t_y,t_z+1} &=& t_z R^{n+1}_{t_x,t_y,t_z-1} + Z_{PQ} R^{n+1}_{t_x,t_y,t_z} \\
# \end{eqnarray}

def R(v_x,v_y,v_z,n,pairAB,pairCD,i,j,alpha,Rpq):

    if(v_x<0 or v_y<0 or v_z<0):
        return 0
    elif(v_x!=0):
        R_np1_vm2 = R(v_x-2,v_y,v_z,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        R_np1_vm1 = R(v_x-1,v_y,v_z,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        return (v_x-1)*R_np1_vm2 + Rpq[0]*R_np1_vm1
    elif(v_y!=0):
        R_np1_vm2 = R(v_x,v_y-2,v_z,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        R_np1_vm1 = R(v_x,v_y-1,v_z,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        return (v_y-1)*R_np1_vm2 + Rpq[1]*R_np1_vm1
    elif(v_z!=0):
        R_np1_vm2 = R(v_x,v_y,v_z-2,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        R_np1_vm1 = R(v_x,v_y,v_z-1,n+1,pairAB,pairCD,i,j,alpha,Rpq)
        return (v_z-1)*R_np1_vm2 + Rpq[2]*R_np1_vm1
    else:
        return (-2*alpha)**n*2*np.pi**(5/2)/(pairAB.exp[i]*pairCD.exp[j]*np.sqrt(pairAB.exp[i]+pairCD.exp[j]))*F_n(n,alpha*np.linalg.norm(Rpq)**2)


# La integral de repulsión electrónica está dada por
# \begin{equation}
# \left(g_{\mathbf{i}} (r,a,A) g_{\mathbf{j}} (r,b,B) | \frac{1}{r_{12}} | g_{\mathbf{k}} (r,c,C)g_{\mathbf{k}} (r,d,D)\right) = \sum_{t_x,t_y,t_z}^{i_x+j_x,i_y+j_y,i_z+j_z} E^{ab}_{t_xt_yt_z} \sum_{u_x,u_y,u_z}^{k_x+l_x,k_y+l_y,k_z+l_z} E_{u_xu_yu_z}^{cd} (-1)^{u_x+u_y+u_z} R_{\mathbf{t}+\mathbf{u}}(\alpha,R_{PQ})
# \end{equation}
# 
# E son las expansiones de Hermite de las distribuciones de traslape
# \begin{equation}
# E_{t}^{ij} = E_{t_x}^{i_x j_x}E_{t_y}^{i_y j_y}E_{t_z}^{i_z j_z}
# \end{equation}
# 
# y R son las integrales de Coulomb de Hermite
# \begin{equation}
# R_{\mathbf{t}}(\alpha,R_{PQ}) = \frac{2 \pi^{5/2}}{pq \sqrt{p+q}} \left(\frac{\partial}{\partial P_x}\right)^{t_x} \left(\frac{\partial}{\partial P_y}\right)^{t_y} \left(\frac{\partial}{\partial P_z}\right)^{t_z} F_0(\alpha R_{PQ}^2)
# \end{equation}
# 

def MD4(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b
    
    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    integral = 0.0
                    for i in range(pairAB.k):
    
                        aux7 = 0
                        for t_z in range(laz+lbz+1): 
                            Ez_ij_t = compute_E(t_z,laz,lbz,pairAB,i,2)
                            aux6 = 0    
                            for t_y in range(lay+lby+1):
                                Ey_ij_t = compute_E(t_y,lay,lby,pairAB,i,1)
                                aux5 = 0        
                                for t_x in range(lax+lbx+1):
                                    Ex_ij_t = compute_E(t_x,lax,lbx,pairAB,i,0)
                                    aux4 = 0
                                    for j in range(pairCD.k):    
                                        alpha = pairAB.exp[i]*pairCD.exp[j]/(pairAB.exp[i]+pairCD.exp[j])
                                        Rpq = pairAB.coord[i]-pairCD.coord[j]
                                        aux3 = 0
                                        for u_z in range(lcz+ldz+1):
                                            Ez_kl_u = compute_E(u_z,lcz,ldz,pairCD,j,2)
                                            aux2 = 0
                                            for u_y in range(lcy+ldy+1):
                                                Ey_kl_u = compute_E(u_y,lcy,ldy,pairCD,j,1)
                                                aux1 = 0
                                                for u_x in range(lcx+ldx+1):
                                                    Ex_kl_u = compute_E(u_x,lcx,ldx,pairCD,j,0)
                                                    aux1 += (-1)**(u_x+u_y+u_z)*Ex_kl_u*R(t_x+u_x,t_y+u_y,t_z+u_z,0,pairAB,pairCD,i,j,alpha,Rpq)
                                                aux2 += Ey_kl_u*aux1
                                            aux3 += Ez_kl_u*aux2
                                        aux4 += aux3*pairCD.coef[j]
                                    aux5 += Ex_ij_t*aux4
                                aux6 += Ey_ij_t*aux5
                            aux7 += Ez_ij_t*aux6
                        integral += aux7*pairAB.coef[i]
                                
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result

# ## MD2

# En este esquema se usa una relación de reucrrencia horizontal para pasar la información del segundo centro al primero, y del cuarto centro al tercero. Posteriormente se obtienen las integrales $(i 0 | k 0)$ a partri de hermitianas con la relación de recurrencia vertical de MD.
# 
# Se usan los coeficientes de expansión
# \begin{equation}
# E_{t}^{i0} = E_{t_x}^{i_x 0}E_{t_y}^{i_y 0}E_{t_z}^{i_z 0}
# \end{equation}

def VRR_MD2(ii,jj,kk,ll,pairAB,pairCD):

    integral = 0.0
    for i in range(pairAB.k):
        aux7 = 0
        for t_z in range(ii[2]+1): 
            Ez_ij_t = compute_E(t_z,ii[2],0,pairAB,i,2)
            aux6 = 0    
            for t_y in range(ii[1]+1):
                Ey_ij_t = compute_E(t_y,ii[1],0,pairAB,i,1)
                aux5 = 0        
                for t_x in range(ii[0]+1):
                    Ex_ij_t = compute_E(t_x,ii[0],0,pairAB,i,0)
                    aux4 = 0
                    for j in range(pairCD.k):    
                        alpha = pairAB.exp[i]*pairCD.exp[j]/(pairAB.exp[i]+pairCD.exp[j])
                        Rpq = pairAB.coord[i]-pairCD.coord[j]
                        aux3 = 0
                        for u_z in range(kk[2]+1):
                            Ez_kl_u = compute_E(u_z,kk[2],0,pairCD,j,2)
                            aux2 = 0
                            for u_y in range(kk[1]+1):
                                Ey_kl_u = compute_E(u_y,kk[1],0,pairCD,j,1)
                                aux1 = 0
                                for u_x in range(kk[0]+1):
                                    Ex_kl_u = compute_E(u_x,kk[0],0,pairCD,j,0)
                                    aux1 += (-1)**(u_x+u_y+u_z)*Ex_kl_u*R(t_x+u_x,t_y+u_y,t_z+u_z,0,pairAB,pairCD,i,j,alpha,Rpq)
                                aux2 += Ey_kl_u*aux1
                            aux3 += Ez_kl_u*aux2
                        aux4 += aux3*pairCD.coef[j]
                    aux5 += Ex_ij_t*aux4
                aux6 += Ey_ij_t*aux5
            aux7 += Ez_ij_t*aux6
        integral += aux7*pairAB.coef[i]
            
    return integral


# y se define la relación de recurrencia horizontal (aquí se expresan las de $X$, pero hay otras equivalentes para $Y$ y $Z$).
# \begin{eqnarray}
# \Theta^N_{i,j+1,k,l} &=& \Theta^N_{i+1,j,k,l} + X_{AB}\Theta_{i,j,k,l}^N\\
# \Theta^N_{i,j,k,l+1} &=& \Theta^N_{i,j,k+1,l} + X_{CD}\Theta_{i,j,k,l}^N
# \end{eqnarray}

def HRR_MD2(i,j,k,l,pairAB,pairCD):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR_MD2(i,j,kp1,lm1,pairAB,pairCD) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])*HRR_MD2(i,j,k,lm1,pairAB,pairCD)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR_MD2(ip1,jm1,k,l,pairAB,pairCD) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR_MD2(i,jm1,k,l,pairAB,pairCD)
    return VRR_MD2(i,j,k,l,pairAB,pairCD)

def MD2(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_MD2(i,j,k,l,pairAB,pairCD)
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result


# # MD1

# Se usa la relación de recurrencia de transferencia de electrones para pasar la información del tercer centro al primer centro.
# 
# \begin{eqnarray}
# \Theta_{i,0,k+1,0}^N &=& - \frac{b X_{AB} + dX_{CD}}{q} \Theta_{i,0,k,0}^{N} + \frac{i}{2q}\Theta^{N}_{i-1,0,k,0} + \frac{k}{2q} \Theta^N_{i,0,k-1,0} - \frac{p}{q} \Theta^N_{i+1,0,k,0}
# \end{eqnarray}

def VRR_MD1(ii,kk,pairAB,pairCD,i,j):

    integral = 0.0
    
    aux7 = 0
    for t_z in range(ii[2]+1): 
        Ez_ij_t = compute_E(t_z,ii[2],0,pairAB,i,2)
        aux6 = 0    
        for t_y in range(ii[1]+1):
            Ey_ij_t = compute_E(t_y,ii[1],0,pairAB,i,1)
            aux5 = 0        
            for t_x in range(ii[0]+1):
                Ex_ij_t = compute_E(t_x,ii[0],0,pairAB,i,0)
                aux4 = 0
                for j in range(pairCD.k):    
                    alpha = pairAB.exp[i]*pairCD.exp[j]/(pairAB.exp[i]+pairCD.exp[j])
                    Rpq = pairAB.coord[i]-pairCD.coord[j]
                    aux3 = 0
                    for u_z in range(kk[2]+1):
                        Ez_kl_u = compute_E(u_z,kk[2],0,pairCD,j,2)
                        aux2 = 0
                        for u_y in range(kk[1]+1):
                            Ey_kl_u = compute_E(u_y,kk[1],0,pairCD,j,1)
                            aux1 = 0
                            for u_x in range(kk[0]+1):
                                Ex_kl_u = compute_E(u_x,kk[0],0,pairCD,j,0)
                                aux1 += (-1)**(u_x+u_y+u_z)*Ex_kl_u*R(t_x+u_x,t_y+u_y,t_z+u_z,0,pairAB,pairCD,i,j,alpha,Rpq)
                            aux2 += Ey_kl_u*aux1
                        aux3 += Ez_kl_u*aux2
                    aux4 += aux3
                aux5 += Ex_ij_t*aux4
            aux6 += Ey_ij_t*aux5
        aux7 += Ez_ij_t*aux6
    integral += aux7
            
    return integral


def trans_elec_MD1(i,j,k,l,pairAB,pairCD,ii,jj):

    integral = 0.0
    
    for xyz in range(3):
        if(i[xyz]<0 or k[xyz]<0):
            return 0
        elif(k[xyz]!=0):
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1 
            km2 = k[:]
            km2[xyz] = km2[xyz] - 2 
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1         
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1        
            return -(pairAB.ab[ii][0]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])+pairCD.ab[jj][1]*(pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz]))/pairCD.exp[jj]*trans_elec_MD1(i,j,km1,l,pairAB,pairCD) + i[xyz]/(2*pairCD.exp[jj])*trans_elec_MD1(im1,j,km1,l,pairAB,pairCD) + (k[xyz]-1)/(2*pairCD.exp[jj])*trans_elec_MD1(i,j,km2,l,pairAB,pairCD) - pairAB.exp[ii]/pairCD.exp[jj]*trans_elec_MD1(ip1,j,km1,l,pairAB,pairCD)
    return VRR_MD1(i,k,pairAB,pairCD,ii,jj)


def HRR_MD1(i,j,k,l,pairAB,pairCD):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR_MD1(i,j,kp1,lm1,pairAB,pairCD) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])*HRR_MD1(i,j,k,lm1,pairAB,pairCD)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR_MD1(ip1,jm1,k,l,pairAB,pairCD) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR_MD1(i,jm1,k,l,pairAB,pairCD)

    integral = 0.0
    for ii in range(pairAB.k):
        aux1 = 0.0
        for jj in range(pairCD.k):
            aux1 += trans_elec_MD1(i,j,k,l,pairAB,pairCD,ii,jj)*pairCD.coef[jj]
        integral += aux1*pairAB.coef[ii]
    return integral

def MD1(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_MD1(i,j,k,l,pairAB,pairCD)
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result



import numpy as np
from .boys import *

# ## OS4
# Se usa la relación de recurrencia
# \begin{eqnarray}
# \Theta_{i+1,j,k,l}^N &=& X_{PA}\Theta_{ijkl}^N - \frac{\alpha}{p}X_{PQ} \Theta_{ijkl}^{N+1} + \frac{i}{2p} \left( \Theta_{i-1,j,k,l}^N - \frac{\alpha}{p} \Theta_{i-1,j,k,l}^{N+1}\right) + \frac{j}{2p} \left( \Theta_{i,j-1,k,l}^N - \frac{\alpha}{p} \Theta_{i,j-1,k,l}^{N+1}\right) + \frac{k}{2(p+q)} \left( \Theta_{i,j,k-1,l}^{N+1}\right) + \frac{l}{2(p+q)} \left( \Theta_{i,j,k,l-1}^{N+1}\right) \\
# \Theta_{i,j+1,k,l}^N &=& X_{PB}\Theta_{ijkl}^N - \frac{\alpha}{p}X_{PQ} \Theta_{ijkl}^{N+1} + \frac{j}{2p} \left( \Theta_{i,j-1,k,l}^N - \frac{\alpha}{p} \Theta_{i,j-1,k,l}^{N+1}\right) + \frac{i}{2p} \left( \Theta_{i-1,j,k,l}^N - \frac{\alpha}{p} \Theta_{i-1,j,k,l}^{N+1}\right) + \frac{k}{2(p+q)} \left( \Theta_{i,j,k-1,l}^{N+1}\right) + \frac{l}{2(p+q)} \left( \Theta_{i,j,k,l-1}^{N+1}\right) \\
# \Theta_{i,j,k+1,l}^N &=& X_{QC}\Theta_{ijkl}^N - \frac{\alpha}{q}X_{PQ} \Theta_{ijkl}^{N+1} + \frac{k}{2q} \left( \Theta_{i,j,k-1,l}^N - \frac{\alpha}{q} \Theta_{i,j,k-1,l}^{N+1}\right) + \frac{l}{2q} \left( \Theta_{i,j,k,l-1}^N - \frac{\alpha}{q} \Theta_{i,j,k,l-1}^{N+1}\right) + \frac{i}{2(p+q)} \left( \Theta_{i-1,j,k,l}^{N+1}\right) + \frac{j}{2(p+q)} \left( \Theta_{i,j-1,k,l}^{N+1}\right) \\
# \Theta_{i,j,k,l+1}^N &=& X_{QD}\Theta_{ijkl}^N - \frac{\alpha}{q}X_{PQ} \Theta_{ijkl}^{N+1} + \frac{l}{2q} \left( \Theta_{i,j,k,l-1}^N - \frac{\alpha}{q} \Theta_{i,j,k,l-1}^{N+1}\right) + \frac{k}{2q} \left( \Theta_{i,j,k-1,l}^N - \frac{\alpha}{q} \Theta_{i,j,k-1,l}^{N+1}\right) + \frac{i}{2(p+q)} \left( \Theta_{i-1,j,k,l}^{N+1}\right) + \frac{j}{2(p+q)} \left( \Theta_{i,j-1,k,l}^{N+1}\right) \\
# \end{eqnarray}

def VRR_OS4(pairAB,pairCD,i,j,k,l,N,ii,jj):

    alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])

    for xyz in range(3):
        if(i[xyz]<0 or j[xyz]<0 or k[xyz]<0 or l[xyz]<0):
            return 0
        elif(i[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            im2 = i[:]
            im2[xyz] = im2[xyz] - 2
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            return (pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])*VRR_OS4(pairAB,pairCD,im1,j,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS4(pairAB,pairCD,im1,j,k,l,N+1,ii,jj) + (i[xyz]-1)/(2*pairAB.exp[ii])*(VRR_OS4(pairAB,pairCD,im2,j,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS4(pairAB,pairCD,im2,j,k,l,N+1,ii,jj)) + (j[xyz])/(2*pairAB.exp[ii])*(VRR_OS4(pairAB,pairCD,i,jm1,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS4(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj)) + (k[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,j,km1,l,N+1,ii,jj)) + (l[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj))
        elif(j[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            jm2 = j[:]
            jm2[xyz] = jm2[xyz] - 2
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            return (pairAB.coord[ii][xyz]-pairAB.g_b.coord[xyz])*VRR_OS4(pairAB,pairCD,i,jm1,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS4(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj) + (j[xyz]-1)/(2*pairAB.exp[ii])*(VRR_OS4(pairAB,pairCD,i,jm2,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS4(pairAB,pairCD,i,jm2,k,l,N+1,ii,jj)) + (i[xyz])/(2*pairAB.exp[ii])*(VRR_OS4(pairAB,pairCD,im1,j,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS4(pairAB,pairCD,im1,j,k,l,N+1,ii,jj)) + (k[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,j,km1,l,N+1,ii,jj)) + (l[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj))
        elif(k[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            km2 = k[:]
            km2[xyz] = km2[xyz] - 2
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            return (pairCD.coord[jj][xyz]-pairCD.g_a.coord[xyz])*VRR_OS4(pairAB,pairCD,i,j,km1,l,N,ii,jj) - alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS4(pairAB,pairCD,i,j,km1,l,N+1,ii,jj) + (k[xyz]-1)/(2*pairCD.exp[jj])*(VRR_OS4(pairAB,pairCD,i,j,km2,l,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS4(pairAB,pairCD,i,j,km2,l,N+1,ii,jj)) + (l[xyz])/(2*pairCD.exp[jj])*(VRR_OS4(pairAB,pairCD,i,j,k,lm1,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS4(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj)) + (i[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,im1,j,k,l,N+1,ii,jj)) + (j[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj))
        elif(k[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            lm2 = l[:]
            lm2[xyz] = lm2[xyz] - 2
            return (pairCD.coord[jj][xyz]-pairCD.g_b.coord[xyz])*VRR_OS4(pairAB,pairCD,i,j,k,lm1,N,ii,jj) - alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS4(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj) + (l[xyz]-1)/(2*pairCD.exp[jj])*(VRR_OS4(pairAB,pairCD,i,j,k,lm2,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS4(pairAB,pairCD,i,j,k,lm2,N+1,ii,jj)) + (k[xyz])/(2*pairCD.exp[jj])*(VRR_OS4(pairAB,pairCD,i,j,km1,l,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS4(pairAB,pairCD,i,j,km1,l,N+1,ii,jj)) + (i[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,im1,j,k,l,N+1,ii,jj)) + (j[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS4(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj))
        
    RAB = np.linalg.norm(pairAB.g_a.coord-pairAB.g_b.coord)
    RCD = np.linalg.norm(pairCD.g_a.coord-pairCD.g_b.coord)
    RPQ = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
    return 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*np.exp(-pairAB.alpha[ii]*RAB**2)*np.exp(-pairCD.alpha[jj]*RCD**2)*F_n(N,alpha*RPQ**2)        


def OS4(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    
                    integral = 0.0
                    for ii in range(pairAB.k):
                        aux1 = 0.0
                        for jj in range(pairCD.k):
                                
                            i = [lax,lay,laz]
                            j = [lbx,lby,lbz]
                            k = [lcx,lcy,lcz]
                            l = [ldx,ldy,ldz]
                            aux1 += VRR_OS4(pairAB,pairCD,i,j,k,l,0,ii,jj)*pairCD.coef[jj]
                        integral += aux1*pairAB.coef[ii]
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result


# ## OS2

# Aquí se aprovecha la relación de recurrencia horizontal.
# 
# Primero se planea construir la parte vertical hacia el primer y tercer centro

def VRR_OS2(pairAB,pairCD,i,j,k,l,N,ii,jj):

    alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])

    for xyz in range(3):
        if(i[xyz]<0 or j[xyz]<0 or k[xyz]<0 or l[xyz]<0):
            return 0
        elif(i[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            im2 = i[:]
            im2[xyz] = im2[xyz] - 2
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            return (pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])*VRR_OS2(pairAB,pairCD,im1,j,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS2(pairAB,pairCD,im1,j,k,l,N+1,ii,jj) + (i[xyz]-1)/(2*pairAB.exp[ii])*(VRR_OS2(pairAB,pairCD,im2,j,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS2(pairAB,pairCD,im2,j,k,l,N+1,ii,jj)) + (j[xyz])/(2*pairAB.exp[ii])*(VRR_OS2(pairAB,pairCD,i,jm1,k,l,N,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS2(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj)) + (k[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS2(pairAB,pairCD,i,j,km1,l,N+1,ii,jj)) + (l[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS2(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj))
        elif(k[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1
            km2 = k[:]
            km2[xyz] = km2[xyz] - 2
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            return (pairCD.coord[jj][xyz]-pairCD.g_a.coord[xyz])*VRR_OS2(pairAB,pairCD,i,j,km1,l,N,ii,jj) - alpha/pairCD.exp[jj]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS2(pairAB,pairCD,i,j,km1,l,N+1,ii,jj) + (k[xyz]-1)/(2*pairCD.exp[jj])*(VRR_OS2(pairAB,pairCD,i,j,km2,l,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS2(pairAB,pairCD,i,j,km2,l,N+1,ii,jj)) + (l[xyz])/(2*pairCD.exp[jj])*(VRR_OS2(pairAB,pairCD,i,j,k,lm1,N,ii,jj) - alpha/pairCD.exp[jj]*VRR_OS2(pairAB,pairCD,i,j,k,lm1,N+1,ii,jj)) + (i[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS2(pairAB,pairCD,im1,j,k,l,N+1,ii,jj)) + (j[xyz])/(2*(pairAB.exp[ii]+pairCD.exp[jj]))*(VRR_OS2(pairAB,pairCD,i,jm1,k,l,N+1,ii,jj))
        
    RAB = np.linalg.norm(pairAB.g_a.coord-pairAB.g_b.coord)
    RCD = np.linalg.norm(pairCD.g_a.coord-pairCD.g_b.coord)
    RPQ = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
    return 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*np.exp(-pairAB.alpha[ii]*RAB**2)*np.exp(-pairCD.alpha[jj]*RCD**2)*F_n(N,alpha*RPQ**2)        


# Posteriormente se usa la horizontal para redirigir hacia el segundo y cuarto centro
# 
# \begin{eqnarray}
# \Theta^N_{i,j+1,k,l} &=& \Theta^N_{i+1,j,k,l} + X_{AB}\Theta_{i,j,k,l}^N\\
# \Theta^N_{i,j,k,l+1} &=& \Theta^N_{i,j,k+1,l} + X_{CD}\Theta_{i,j,k,l}^N
# \end{eqnarray}

def HRR_OS2(pairAB,pairCD,i,j,k,l,N):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR_OS2(pairAB,pairCD,i,j,kp1,lm1,N) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz])*HRR_OS2(pairAB,pairCD,i,j,k,lm1,N)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR_OS2(pairAB,pairCD,ip1,jm1,k,l,N) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR_OS2(pairAB,pairCD,i,jm1,k,l,N)
    
    integral = 0.0
    for ii in range(pairAB.k):
        aux1 = 0.0
        for jj in range(pairCD.k):
                    
            aux1 += VRR_OS2(pairAB,pairCD,i,j,k,l,N,ii,jj)*pairCD.coef[jj]
        integral += aux1*pairAB.coef[ii]    
    
    return integral


def OS2(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                                              
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_OS2(pairAB,pairCD,i,j,k,l,0)
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result


# ## OS1

# Se define la relación de recurrencia de Obara-Saika (aquí se expresan las de $X$, pero hay otras equivalentes para $Y$ y $Z$).
# \begin{eqnarray}
# \Theta_{i+1,0,0,0}^N &=& X_{PA}\Theta_{i000}^N - \frac{\alpha}{p}X_{PQ} \Theta_{i000}^{N+1} + \frac{i}{2p} \left( \Theta_{i-1,0,0,0}^N - \frac{\alpha}{p} \Theta_{i-1,0,0,0}^{N+1}\right)
# \end{eqnarray}

def VRR_OS1(i,j,k,l,N,pairAB,pairCD,ii,jj):

    alpha = pairAB.exp[ii]*pairCD.exp[jj]/(pairAB.exp[ii]+pairCD.exp[jj])

    for xyz in range(3):
        if(i[xyz]<0):
            return 0
        elif(i[xyz]>0):
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1
            im2 = i[:]
            im2[xyz] = im2[xyz] - 2
            return (pairAB.coord[ii][xyz]-pairAB.g_a.coord[xyz])*VRR_OS1(im1,j,k,l,N,pairAB,pairCD,ii,jj) - alpha/pairAB.exp[ii]*(pairAB.coord[ii][xyz]-pairCD.coord[jj][xyz])*VRR_OS1(im1,j,k,l,N+1,pairAB,pairCD,ii,jj) + (i[xyz]-1)/(2*pairAB.exp[ii])*(VRR_OS1(im2,j,k,l,N,pairAB,pairCD,ii,jj) - alpha/pairAB.exp[ii]*VRR_OS1(im2,j,k,l,N+1,pairAB,pairCD,ii,jj))
        
    RAB = np.linalg.norm(pairAB.g_a.coord-pairAB.g_b.coord)
    RCD = np.linalg.norm(pairCD.g_a.coord-pairCD.g_b.coord)
    RPQ = np.linalg.norm(pairAB.coord[ii]-pairCD.coord[jj])
    return 2*np.pi**(5/2)/(pairAB.exp[ii]*pairCD.exp[jj]*np.sqrt(pairAB.exp[ii]+pairCD.exp[jj]))*np.exp(-pairAB.alpha[ii]*RAB**2)*np.exp(-pairCD.alpha[jj]*RCD**2)*F_n(N,alpha*RPQ**2)                


# Se define la relación de recurrencia de Obara-Saika para transferencia de electrones (aquí se expresan las de $X$, pero hay otras equivalentes para $Y$ y $Z$).
# \begin{eqnarray}
# \Theta_{i,0,k+1,0}^N &=& - \frac{b X_{AB} + dX_{CD}}{q} \Theta_{i,0,k,0}^{N} + \frac{i}{2q}\Theta^{N}_{i-1,0,k,0} + \frac{k}{2q} \Theta^N_{i,0,k-1,0} - \frac{p}{q} \Theta^N_{i+1,0,k,0}
# \end{eqnarray}

def trans_elec_OS1(i,j,k,l,N,pairAB,pairCD,ii,jj):

    integral = 0.0
    
    for xyz in range(3):
        if(i[xyz]<0 or k[xyz]<0):
            return 0
        elif(k[xyz]!=0):
            km1 = k[:]
            km1[xyz] = km1[xyz] - 1 
            km2 = k[:]
            km2[xyz] = km2[xyz] - 2 
            im1 = i[:]
            im1[xyz] = im1[xyz] - 1         
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1        
            return -(pairAB.ab[ii][0]*(pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])+pairCD.ab[jj][1]*(pairCD.g_c.coord[xyz]-pairCD.g_d.coord[xyz]))/pairCD.exp[jj]*trans_elec_OS1(i,j,km1,l,N,pairAB,pairCD) + i[xyz]/(2*pairCD.exp[jj])*trans_elec_OS1(im1,j,km1,l,N,pairAB,pairCD) + (k[xyz]-1)/(2*pairCD.exp[jj])*trans_elec_OS1(i,j,km2,l,N,pairAB,pairCD) - pairAB.exp[ii]/pairCD.exp[jj]*trans_elec_OS1(ip1,j,km1,l,N,pairAB,pairCD)
    return VRR_OS1(i,j,k,l,N,pairAB,pairCD,ii,jj)


# Se define la relación de recurrencia horizontal (aquí se expresan las de $X$, pero hay otras equivalentes para $Y$ y $Z$).
# \begin{eqnarray}
# \Theta^N_{i,j+1,k,l} &=& \Theta^N_{i+1,j,k,l} + X_{AB}\Theta_{i,j,k,l}^N\\
# \Theta^N_{i,j,k,l+1} &=& \Theta^N_{i,j,k+1,l} + X_{CD}\Theta_{i,j,k,l}^N
# \end{eqnarray}

def HRR_OS1(i,j,k,l,N,pairAB,pairCD):

    for xyz in range(3):
        if(l[xyz]!=0):
            lm1 = l[:]
            lm1[xyz] = lm1[xyz] - 1
            kp1 = k[:]
            kp1[xyz] = kp1[xyz] + 1
            return HRR(i,j,kp1,lm1,pairAB,pairCD,N) + (pairCD.g_a.coord[xyz]-pairCD.g_b.coord[xyz],0)*HRR(i,j,k,lm1,pairAB,pairCD,N)
        elif(j[xyz]!=0):
            jm1 = j[:]
            jm1[xyz] = jm1[xyz] - 1
            ip1 = i[:]
            ip1[xyz] = ip1[xyz] + 1
            return HRR(ip1,jm1,k,l,pairAB,pairCD,N) + (pairAB.g_a.coord[xyz]-pairAB.g_b.coord[xyz])*HRR(i,jm1,k,l,pairAB,pairCD,N)

    integral = 0.0
    for ii in range(pairAB.k):
        aux1 = 0.0
        for jj in range(pairCD.k):
            aux1 += trans_elec_OS1(i,j,k,l,N,pairAB,pairCD,ii,jj)*pairCD.coef[jj]
        integral += aux1*pairAB.coef[ii]
    return integral


# La integral de repulsión electrónica está dada por
# \begin{equation}
# \left(g_{\mathbf{i}} (r,a,A) g_{\mathbf{j}} (r,b,B) | \frac{1}{r_{12}} | g_{\mathbf{k}} (r,c,C)g_{\mathbf{k}} (r,d,D)\right)
# \end{equation}
# 
# **Tip.** Para mayor eficiencia, evaluar todas las F_n desde 0 hasta |i|+|j|+|k|+|l| al inicio y guardarlas en un vector.

def OS1(pairAB,pairCD):

    g_a = pairAB.g_a
    g_b = pairAB.g_b
    g_c = pairCD.g_a
    g_d = pairCD.g_b

    result = np.zeros((len(g_a.orientaciones),len(g_b.orientaciones),len(g_c.orientaciones),len(g_d.orientaciones)))
    for a,lax,lay,laz in g_a.orientaciones:
        for b,lbx,lby,lbz in g_b.orientaciones:
            for c,lcx,lcy,lcz in g_c.orientaciones:
                for d,ldx,ldy,ldz in g_d.orientaciones:
                    
                    i = [lax,lay,laz]
                    j = [lbx,lby,lbz]
                    k = [lcx,lcy,lcz]
                    l = [ldx,ldy,ldz]
                    integral = HRR_OS1(i,j,k,l,0,pairAB,pairCD)
                    
                    result[a][b][c][d] = g_a.N[a]*g_b.N[b]*g_c.N[c]*g_d.N[d]*integral
    return result



import numpy as np
from scipy.special import factorial2
import pubchempy as pcp

try:
    import importlib.resources as pkg_resources
except ImportError:
    import importlib_resources as pkg_resources
from . import basis


class shell:
    def __init__(self,l,exp,coef,coord):
        self.l = l
        self.exp = exp
        self.coef = coef
        self.coord = np.array(coord)
        self.k = len(exp)
        
        #normalizacion
        for i in range(self.k):
            coef[i] = coef[i]*(2*exp[i]/np.pi)**(3/4)*(4*exp[i])**(l/2)
            
        norm = 0
        for j in range(self.k):
            for k in range(self.k):
                norm += coef[j]*coef[k]*(np.pi/(exp[j]+exp[k]))**(3/2)/(2*(exp[j]+exp[k]))**(l)

        for i in range(self.k):
            coef[i] = coef[i]/np.sqrt(norm)
        
        self.orientaciones = []
        i = 0
        for lx in range(l,-1,-1):
            for ly in range(l-lx,-1,-1):
                lz = l-lx-ly
                self.orientaciones.append([i,lx,ly,lz])      
                i = i + 1

        self.N = np.ones(int((l+1)*(l+2)/2))
        for i,lx,ly,lz in self.orientaciones:
            self.N[i] = np.sqrt(1/(factorial2(2*lx-1)*factorial2(2*ly-1)*factorial2(2*lz-1)))

class shell_pair:
    def __init__(self,g_a,g_b):
        
        self.g_a = g_a
        self.g_b = g_b
        
        self.exp = np.add.outer(g_a.exp, g_b.exp).flatten()
        self.coef = np.multiply.outer(g_a.coef, g_b.coef).flatten()
        
        self.coord = []
        self.ab = []
        for i in range(g_a.k):
            for j in range(g_b.k):
                self.coord.append((g_a.exp[i]*g_a.coord+g_b.exp[j]*g_b.coord)/(g_a.exp[i]+g_b.exp[j]))
                self.ab.append((g_a.exp[i],g_b.exp[j]))

                
        self.alpha = []
        for i in range(g_a.k):
            for j in range(g_b.k):
                self.alpha.append((g_a.exp[i]*g_b.exp[j])/(g_a.exp[i]+g_b.exp[j]))
                                
        self.k = len(self.exp)

def get_set_of_shells(element,coord,basis_name):

    element = element.upper()
    shells = []
    
    # Open basis file
    f = pkg_resources.open_text(basis, basis_name+".bas")    
    #f = open("basis/"+basis_name+".bas","r")

    # Find_atom
    atom_found = False
    for line in f:
        if(atom_found):
            break
        splitted_line = line.split()
        if(len(splitted_line)>0):
            if("O-" in splitted_line[0]):
                if(splitted_line[1] == element):
                    atom_found=True

    # Pass the scheme contraction info line
    for line in f:
        num_of_shells = int(line)
        break
    
    # Read shells
    new_shell = True
    idx_shell = 0
    idx_prim = 0
    for line in f:
        splitted_line = line.split()
        #If new shell, read angular momentum and number of primitives
        if new_shell:
            idx_shell += 1
            idx_prim = 0        
            idx,l,num_of_primitives = int(splitted_line[0]),int(splitted_line[1]),int(splitted_line[2])
            exp = []
            coef = []
            new_shell = False
            continue
        #If not a new shell, read exponents and coefficients
        if not new_shell:
            idx_prim += 1
            exp.append(float(splitted_line[0]))
            coef.append(float(splitted_line[1]))
            if(idx_prim == num_of_primitives):
                shells.append(shell(l,exp,coef,coord))
                #If all shells readed, break, else, read new shell
                if(idx_shell == num_of_shells):
                    break
                else:
                    new_shell = True
            continue
    f.close()
    
    return shells       

def get_all_shells_from_pubchempy_name(name,basis_name):
    c = pcp.get_compounds(name, 'name')
    cid = c.cid
    c = pcp.Compound.from_cid(cid,record_type='3d')
    dic = c.to_dict(properties=['atoms'])
    shells = []
    for atom in dic['atoms']:
        shells += get_set_of_shells(atom['element'],(atom['x'],atom['y'],atom['z']),basis_name)
    return shells

def get_all_shells_from_pubchempy_cid(cid,basis_name):
    c = pcp.Compound.from_cid(cid,record_type='3d')
    dic = c.to_dict(properties=['atoms'])
    shells = []
    for atom in dic['atoms']:
        shells += get_set_of_shells(atom['element'],(atom['x'],atom['y'],atom['z']),basis_name)
    return shells

def get_all_shells_from_xyz(molecule,basis_name):
    molecule = molecule.split()
    natoms = int(len(molecule)/4)
    shells = []
    for atom in range(natoms):
        shells += get_set_of_shells(molecule[atom*4],(float(molecule[atom*4+1]),float(molecule[atom*4+2]),float(molecule[atom*4+3])),basis_name)
    return shells



import setuptools

with open("READMEpy.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="GalletaPy",
    version="0.12",
    author="Juan Felipe Huan Lew Yee",
    author_email="felipe.lew.yee@gmail.com",
    description="Paquete con código para ERIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/felipelewyee/galleta",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    package_data={'': ['basis/*.bas']},
    include_package_data=True,    
    install_requires=['numpy','scipy','pubchempy'],
)

